@@ -14,13 +14,11 @@
 #' @param fitsTest_settings List of arguments. It must contain the following elements.
 #'                         \itemize{
 #'                              \item \strong{subsamples}: Named list of boolean vectors. Each element of the list defines whether a given observation belongs to a given subsample (e.g. by sociodemographics).
-#'                              \item \strong{modelComponent}: Name of model component. Set to model by default.
 #'                         }
 #' @return Matrix with average log-likelihood at observation level per category (invisibly).
 #' @export
 apollo_fitsTest=function(model,apollo_probabilities,apollo_inputs,fitsTest_settings){
   
-  if(is.null(fitsTest_settings[["modelComponent"]])) fitsTest_settings$modelComponent="model"
   if(is.null(fitsTest_settings[["subsamples"]])) fitsTest_settings[["subsamples"]]=NULL
   
   fits=model$avgLL
