@@ -15,9 +15,10 @@
 #'                         \item \strong{\code{budget}}: Numeric vector. Budget for each observation.
 #'                         \item \strong{\code{minConsumption}}: Named list of scalars or numeric vectors. Minimum consumption of the alternatives, if consumed. As many elements as alternatives. Names must match those in \code{alternatives}.
 #'                         \item \strong{\code{outside}}: Character. Optional name of the outside good.
-#'                         \item \strong{\code{rows}}: Boolean vector. Consideration of rows in the likelihood calculation, FALSE to exclude. Length equal to the number of observations (nObs). Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
+#'                         \item \strong{\code{rows}}: Logical vector. Consideration of rows in the likelihood calculation, FALSE to exclude. Length equal to the number of observations (nObs). Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
 #'                         \item \strong{\code{componentName}}: Character. Name given to model component.
 #'                         \item \strong{\code{nRep}}: Numeric scalar. Number of simulations of the whole dataset used for forecasting. The forecast is the average of these simulations. Default is 100.
+#'                         \item \strong{\code{rawPrediction}}: Logical scalar. TRUE for prediction to be returned at the draw level (a 3-dim array). FALSE for prediction to be returned averaged across draws. Default is FALSE.
 #'                       }
 #' @param functionality Character. Can take different values depending on desired output.
 #'                      \itemize{
@@ -240,11 +241,11 @@ apollo_mdcev <- function(mdcev_settings,functionality){
   ### Deal with outside
   if(mdcev_settings$hasOutside){
     # Add gamma outside, if missing
-    if(is.null(mdcev_settings$gamma$outside)) mdcev_settings$gamma$outside <- 1
+    if(is.null(mdcev_settings$gamma[[mdcev_settings$outside]])) mdcev_settings$gamma[[mdcev_settings$outside]] <- 1
     # Replace mdcev_settings$outside by "outside" in V, alpha, and gamma
-    tmp <- which(names(mdcev_settings$V    )==mdcev_settings$output); if(length(tmp)>0) names(mdcev_settings$V    )[tmp] <- "outside"
-    tmp <- which(names(mdcev_settings$alpha)==mdcev_settings$output); if(length(tmp)>0) names(mdcev_settings$alpha)[tmp] <- "outside"
-    tmp <- which(names(mdcev_settings$gamma)==mdcev_settings$output); if(length(tmp)>0) names(mdcev_settings$gamma)[tmp] <- "outside"
+    tmp <- which(names(mdcev_settings$V    )==mdcev_settings$outside); if(length(tmp)>0) names(mdcev_settings$V    )[tmp] <- "outside"
+    tmp <- which(names(mdcev_settings$alpha)==mdcev_settings$outside); if(length(tmp)>0) names(mdcev_settings$alpha)[tmp] <- "outside"
+    tmp <- which(names(mdcev_settings$gamma)==mdcev_settings$outside); if(length(tmp)>0) names(mdcev_settings$gamma)[tmp] <- "outside"
     rm(tmp)
   }
   ### Reorder V, alpha and gamma if necessary
@@ -308,10 +309,11 @@ apollo_mdcev <- function(mdcev_settings,functionality){
     rm(mdcev_settings)
     
     # Check that sigma is not random
-    if(!is.vector(s$sigma)) stop("Forecasting not available for ")
+    if(!is.vector(s$sigma)) stop("Forecasting not available for MDCEV with random sigma")
     
     # Generate draws for Gumbel error components
-    set.seed(99)
+    if(!is.null(apollo_inputs$apollo_control$seed)) seed <- apollo_inputs$apollo_control$seed + 5 else seed <- 13 + 5
+    set.seed(seed)
     tmp1 <- -log(-log(apollo_mlhs(s$nRep, s$nAlt, s$nObs)))
     epsL <- vector(mode="list", length=s$nRep)
     tmp2 <- (0:(s$nObs-1))*s$nRep
@@ -319,12 +321,14 @@ apollo_mdcev <- function(mdcev_settings,functionality){
     rm(tmp1, tmp2)
     
     # Initialise output matrices
-    Xm <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
-    Xv <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
-    Mm <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
-    Mv <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
-    Em <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
-    Ev <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
+    if(s$rawPrediction) XX <- array(NA, dim=c(s$nObs, s$nAlt, s$nRep), dimnames=list(NULL, s$alternatives, NULL)) else {
+      Xm <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
+      Xv <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
+      Mm <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
+      Mv <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
+      Em <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
+      Ev <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
+    }
     X  <- matrix(0, nrow=s$nObs, ncol=s$nAlt)
     
     # Tools to deal with draws
@@ -354,6 +358,7 @@ apollo_mdcev <- function(mdcev_settings,functionality){
     # Expand avail and cost
     avail <- sapply(s$avail, function(a) if(length(a)==1) rep(a, s$nObs) else a)
     cost  <- sapply(s$cost , function(x) if(length(x)==1) rep(x, s$nObs) else x)
+    if(length(s$budget)==1) budget <- rep(s$budget, s$nObs) else budget <- s$budget
     
     ### Simulate prediction
     if(!is.null(apollo_inputs$silent)) silent <- apollo_inputs$silent else silent <- FALSE
@@ -372,7 +377,7 @@ apollo_mdcev <- function(mdcev_settings,functionality){
             # Calculate prediction
             for(i in 1:s$nObs){
               p  <- cost[i,2:s$nAlt]
-              b  <- s$budget[i]
+              b  <- budget[i]
               g  <- gamma[i,2:s$nAlt]
               a0 <- alpha[i,1]
               ak <- alpha[i,2:s$nAlt]
@@ -404,12 +409,14 @@ apollo_mdcev <- function(mdcev_settings,functionality){
         } # end of "for" loop over inter draws
         X <- X/nInter # CHECK IF THIS IS CORRECT!!!
         # Store prediction (maybe this needs to be nested in the for loops in a more clever way to address for draws)
-        Xm <- Xm + X/s$nRep
-        Mm <- Mm + (X>0)/s$nRep
-        Em <- Em + X*cost/s$nRep
-        Xv <- Xv + apply(X  , MARGIN=2, function(v) (v-mean(v))^2)/s$nRep
-        Mv <- Mv + apply(X>0, MARGIN=2, function(m) (m-mean(m))^2)/s$nRep
-        Ev <- Ev + apply(X*cost, MARGIN=2, function(e) (e-mean(e))^2)/s$nRep
+        if(s$rawPrediction) XX[,,r] <- X else {
+          Xm <- Xm + X/s$nRep
+          Mm <- Mm + (X>0)/s$nRep
+          Em <- Em + X*cost/s$nRep
+          Xv <- Xv + apply(X  , MARGIN=2, function(v) (v-mean(v))^2)/s$nRep
+          Mv <- Mv + apply(X>0, MARGIN=2, function(m) (m-mean(m))^2)/s$nRep
+          Ev <- Ev + apply(X*cost, MARGIN=2, function(e) (e-mean(e))^2)/s$nRep
+        }
         if(!silent){ if(r%%step2==0) cat(round(100*r/s$nRep,0), "%", sep="") else { if(r%%step1==0) cat(".") } }
       } # end of "for" loop over repetitions
     } else {
@@ -427,7 +434,7 @@ apollo_mdcev <- function(mdcev_settings,functionality){
             # Calculate prediction
             for(i in 1:s$nObs){
               p  <- cost[i,]
-              b  <- s$budget[i]
+              b  <- budget[i]
               g  <- gamma[i,]
               ak <- alpha[i,]
               phk<- phiP[i,] 
@@ -453,22 +460,26 @@ apollo_mdcev <- function(mdcev_settings,functionality){
         } # end of "for" loop over inter draws
         X <- X/nInter
         # Store prediction (maybe this needs to be nested in the for loops in a more clever way to address for draws)
-        Xm <- Xm + X/s$nRep
-        Mm <- Mm + (X>0)/s$nRep
-        Em <- Em + X*cost/s$nRep
-        Xv <- Xv + apply(X  , MARGIN=2, function(v) (v-mean(v))^2)/s$nRep
-        Mv <- Mv + apply(X>0, MARGIN=2, function(m) (m-mean(m))^2)/s$nRep
-        Ev <- Ev + apply(X*cost, MARGIN=2, function(e) (e-mean(e))^2)/s$nRep
+        if(s$rawPrediction) XX[,,r] <- X else {
+          Xm <- Xm + X/s$nRep
+          Mm <- Mm + (X>0)/s$nRep
+          Em <- Em + X*cost/s$nRep
+          Xv <- Xv + apply(X  , MARGIN=2, function(v) (v-mean(v))^2)/s$nRep
+          Mv <- Mv + apply(X>0, MARGIN=2, function(m) (m-mean(m))^2)/s$nRep
+          Ev <- Ev + apply(X*cost, MARGIN=2, function(e) (e-mean(e))^2)/s$nRep
+        }
         if(!silent){ if(r%%step2==0) cat(round(100*r/s$nRep,0), "%", sep="") else { if(r%%step1==0) cat(".") } }
       } # end of "for" loop over repetitions
     }
     if(!silent) cat("\n")
     
     ### Prepare output
-    out <- cbind(Xm, sqrt(Xv), Mm, sqrt(Mv), Em, sqrt(Ev))
-    out <- apollo_insertRows(out, s$rows, NA)
-    colN <- c("cont_mean", "cont_sd", "disc_mean", "disc_sd", "expe_mean", "expe_sd")
-    colnames(out) <- paste( names(s$continuousChoice), rep(colN, each=s$nAlt), sep="_")
+    if(s$rawPrediction) out <- XX else {
+      out <- cbind(Xm, sqrt(Xv), Mm, sqrt(Mv), Em, sqrt(Ev))
+      out <- apollo_insertRows(out, s$rows, NA)
+      colN <- c("cont_mean", "cont_sd", "disc_mean", "disc_sd", "expe_mean", "expe_sd")
+      colnames(out) <- paste( names(s$continuousChoice), rep(colN, each=s$nAlt), sep="_")
+    }
     return(out)
     
   }
