@@ -9,6 +9,7 @@
 #' @param  d The number of dimensions to generate draws in
 #' @param  i The number of individuals to generate draws for
 #' @return A (N*i) x d matrix with random draws
+#' @export
 apollo_mlhs=function(N,d,i){
   shuffle=function(inv){
     out=inv[rank(stats::runif(length(inv)))];
@@ -16,15 +17,10 @@ apollo_mlhs=function(N,d,i){
   
   temp=seq(0,N-1)/N;
   out=matrix(0,N*i,d);
-  j=1;
-  k=1;
-  while(k<d+1){
-    j=1;
-    while(j<i+1){
+  for(k in 1:d){
+    for(j in 1:i){
       out[(1+N*(j-1)):(N*j),k] = shuffle(temp + stats::runif(1)/N);
-      j=j+1
     }
-    k=k+1
   }
   out
 }
\ No newline at end of file
