@@ -82,6 +82,8 @@
 #' @param silent Logical. TRUE to keep the function from printing to the console. Default is FALSE.
 #' @return List grouping several required input for model estimation.
 #' @export
+#' @importFrom tibble is_tibble
+#' @importFrom utils installed.packages
 apollo_validateInputs <- function(apollo_beta=NA, apollo_fixed=NA, database=NA,
                                   apollo_control=NA, 
                                   apollo_HB=NA, apollo_draws=NA,
@@ -117,6 +119,9 @@ apollo_validateInputs <- function(apollo_beta=NA, apollo_fixed=NA, database=NA,
   }
   if(all(names(apollo_beta) %in% apollo_fixed)) stop('All elements in apollo_beta are included in apollo_fixed, so there is nothing to estimate!')
   
+  ### If database is a tibble, turn it into a data.frame
+  if('tibble' %in% installed.packages()[,"Package"] && tibble::is_tibble(database)) database <- as.data.frame(database)
+  
   ### Validate apollo_control, database
   apollo_control <- apollo_validateControl(database, apollo_control, silent=silent)
   database       <- apollo_validateData(database, apollo_control, silent=silent)
